package svc

import (
	cfg2 "cicv-data-closedloop/kinglong/common/cfg"
	"cicv-data-closedloop/kinglong/common/cutil"
	"cicv-data-closedloop/kinglong/common/log"
	"cicv-data-closedloop/kinglong/common/util"
	"github.com/bluenviron/goroslib/v2"
	"os"
	"strconv"
	"time"
)

// BagRecord 打包rosbag
func BagRecord(nodeName string) {
	log.GlobalLogger.Info("rosbag record goroutine - 启动")
	for {
		log.GlobalLogger.Info("校验必需的 rosnode 是否全部启动。")
		canRecord := false
		for !canRecord {
			time.Sleep(time.Duration(2) * time.Second)
			canRecord = isCanRecord(cfg2.RosNode)
		}
		log.GlobalLogger.Info("rosnode启动完成，正在启动record命令。")

		var command []string
		command = append(command, "record")
		command = append(command, "--split")
		command = append(command, "--duration=1")
		for _, host := range cfg2.CloudConfig.Hosts {
			if host.Name == nodeName {
				for _, topic := range host.Topics {
					command = append(command, topic)
				}
			}
		}

		// 2 ------- 调用 rosbag 打包命令，该命令自动阻塞 -------
		// 不在此处压缩，因为 rosbag filter 时会报错。在上传到oss之前压缩即可。
		// 包名格式：2023-11-15-17-35-20_0.bag
		cutil.CreateParentDir(cfg2.CloudConfig.BagDataDir)
		cmd, err := util.ExecuteWithPath(cfg2.CloudConfig.BagDataDir, "rosbag", command...)
		if err != nil {
			log.GlobalLogger.Error("执行record命令", command, "出错：", err)
			continue
		}
		log.GlobalLogger.Info("启动record命令成功。")

		recordProcessPid := strconv.Itoa(cmd.Process.Pid)
		var recordSubProcessPid string
		for {
			time.Sleep(time.Duration(2) * time.Second)
			recordSubProcessPid, err = util.GetSubProcessPid(recordProcessPid)
			if err != nil {
				log.GlobalLogger.Info("正在等待获取进程 ", recordProcessPid, " 的子进程的pid。")
				continue
			}
			if recordSubProcessPid != "" {
				log.GlobalLogger.Info("获取进程 ", recordProcessPid, " 的子进程的pid：", recordSubProcessPid)
				break
			}
		}
		// 等待自杀信号
		log.GlobalLogger.Info("rosbag record goroutine - 等待自杀信号")
		select {
		case signal := <-ChannelKillRosRecord:
			if signal == 1 {
				err = cmd.Process.Kill()
				if err != nil {
					log.GlobalLogger.Error("程序崩溃，杀死record命令进程", recordProcessPid, "出错：", err)
					os.Exit(-1)
				}
				err = util.KillProcessByPID(recordSubProcessPid)
				if err != nil {
					log.GlobalLogger.Error("程序崩溃，杀死record命令子进程", recordSubProcessPid, "出错：", err)
					os.Exit(-1)
				}
				AddKillTimes("1")
				return
			}
		}

		// TODO 暂时不放开该逻辑。如果监控rosnode来判断是否杀死record，太麻烦，生成窗口的线程也需要关闭
		//{
		//	commonCfg.GlobalLogger.Info("正在监控rosnode是否全部关闭。")
		//	for canRecord {
		//		time.Sleep(time.Duration(1) * time.Second)
		//		canRecord = isCanRecord(commonCfg.RosNode)
		//	}
		//	commonCfg.GlobalLogger.Info("rosnode已全部关闭，正在结束record进程。")
		//
		//	err = cmd.Process.Kill()
		//	if err != nil {
		//		commonCfg.GlobalLogger.Error("杀死record进程错误：", err)
		//		continue
		//	}
		//}
	}
}

func isCanRecord(n *goroslib.Node) bool {
	time.Sleep(time.Duration(1) * time.Second)
	nodes, err := n.MasterGetNodes()
	if err != nil {
		log.GlobalLogger.Error("获取rosnode出错：", err)
		return false
	}
	myMap := nodes
	mySlice := cfg2.CloudConfig.Ros.Nodes
	for _, element := range mySlice {
		if _, ok := myMap[element]; !ok {
			log.GlobalLogger.Info("rosnode：", element, " 未启动，需等待启动后才可启动record。")
			return false
		}
	}
	return true
}
